"""
Domain

This module contains actions related to Active Directory domain configuration & policies.
"""
import contextlib

from ppa import ui
from active_directory import domain, _translators, exceptions as sdk_exceptions

from ._wrappers import ldap_function
from .records import DomainController


@contextlib.contextmanager
def _catch_domain_object_errors():
    try:
        yield
    except sdk_exceptions.NoSuchObject as e:
        ui.output_error(str(e))
        raise e


@ldap_function
def maximum_password_age(domain_controller: DomainController) -> _translators.TimeUnits:
    """Get the [Maximum Password Age](https://docs.microsoft.com/en-us/previous-versions/windows/it-pro/windows-server-2012-r2-and-2012/hh994573(v=ws.11)) policy setting from the domain.

!!! question "Minimum Plugin Version: 6.3.0"

??? info "Input"

    - __domain_controller:__ a [DomainController](../inputs_outputs/#domaincontroller) dictionary

??? success "Output"

    A single [TimePeriod](../inputs_outputs/#timeperiod).

??? example
    {% raw %}
    ```YAML linenums="1"
    - active_directory.domain.maximum_password_age:
      load:
        domain_controller: domain_controller
      save: maximum_password_age

    - ppa.ui.output_info:
        text: The maximum password age is {{ maximum_password_age.days }} days & {{ maximum_password_age.seconds }} seconds.
    ```
    {% endraw %}
"""
    with _catch_domain_object_errors():
        return domain.get_max_password_age(domain_controller)


@ldap_function
def minimum_password_age(domain_controller: DomainController) -> _translators.TimeUnits:
    """Get the [Minimum Password Age](https://docs.microsoft.com/en-us/previous-versions/windows/it-pro/windows-server-2012-r2-and-2012/hh994570(v=ws.11)) policy setting from the domain.

!!! question "Minimum Plugin Version: 6.3.0"

??? info "Input"

    - __domain_controller:__ a [DomainController](../inputs_outputs/#domaincontroller) dictionary

??? success "Output"

    A single [TimePeriod](../inputs_outputs/#timeperiod).

??? example
    {% raw %}
    ```YAML linenums="1"
    - active_directory.domain.minimum_password_age:
      load:
        domain_controller: domain_controller
      save: minimum_password_age

    - ppa.ui.output_info:
        text: The minimum password age is {{ minimum_password_age.days }} days & {{ minimum_password_age.seconds }} seconds.
    ```
    {% endraw %}
"""
    with _catch_domain_object_errors():
        return domain.get_min_password_age(domain_controller)


@ldap_function
def password_history_length(domain_controller: DomainController) -> int:
    """Get the [Enforce Password History](https://docs.microsoft.com/en-us/previous-versions/windows/it-pro/windows-server-2012-r2-and-2012/hh994571(v=ws.11)) policy setting from the domain.

!!! question "Minimum Plugin Version: 6.3.0"

??? info "Input"

    - __domain_controller:__ a [DomainController](../inputs_outputs/#domaincontroller) dictionary

??? success "Output"

    The password history length as a number.

??? example
    {% raw %}
    ```YAML linenums="1"
    - active_directory.domain.password_history_length:
      load:
        domain_controller: domain_controller
      save: history_length

    - ppa.ui.output_info:
        text: New passwords must differ from the last {{ history_length }} passwords.
    ```
    {% endraw %}
"""
    with _catch_domain_object_errors():
        return domain.get_password_history_length(domain_controller)


@ldap_function
def minimum_password_length(domain_controller: DomainController) -> int:
    """Get the [Minimum Password Length](https://docs.microsoft.com/en-us/previous-versions/windows/it-pro/windows-server-2012-r2-and-2012/hh994560(v=ws.11)) policy setting from the domain.

!!! question "Minimum Plugin Version: 6.3.0"

??? info "Input"

    - __domain_controller:__ a [DomainController](../inputs_outputs/#domaincontroller) dictionary

??? success "Output"

    The minimum password length as a number.

??? example
    {% raw %}
    ```YAML linenums="1"
    - active_directory.domain.minimum_password_length:
      load:
        domain_controller: domain_controller
      save: minimum_length

    - ppa.ui.output_info:
        text: New passwords must be at least {{ minimum_length }} characters long.
    ```
    {% endraw %}
"""
    with _catch_domain_object_errors():
        return domain.get_minimum_password_length(domain_controller)


@ldap_function
def lockout_threshold(domain_controller: DomainController) -> int:
    """Get the [Account Lockout Threshold](https://docs.microsoft.com/en-us/previous-versions/windows/it-pro/windows-server-2012-r2-and-2012/hh994574(v=ws.11)) policy setting from the domain.

!!! question "Minimum Plugin Version: 6.3.0"

??? info "Input"

    - __domain_controller:__ a [DomainController](../inputs_outputs/#domaincontroller) dictionary

??? success "Output"

    The account lockout threshold as a number.

??? example
    {% raw %}
    ```YAML linenums="1"
    - active_directory.domain.lockout_threshold:
      load:
        domain_controller: domain_controller
      save: lockout_threshold

    - ppa.ui.output_info:
        text: Accounts will be locked out after {{ lockout_threshold }} failed logon attempts.
    ```
    {% endraw %}
"""
    with _catch_domain_object_errors():
        return domain.get_lockout_threshold(domain_controller)


@ldap_function
def lockout_duration(domain_controller: DomainController) -> _translators.TimeUnits:
    """Get the [Account Lockout Duration](https://docs.microsoft.com/en-us/previous-versions/windows/it-pro/windows-server-2012-r2-and-2012/hh994569(v=ws.11)) policy setting from the domain.

!!! question "Minimum Plugin Version: 6.3.0"

??? info "Input"

    - __domain_controller:__ a [DomainController](../inputs_outputs/#domaincontroller) dictionary

??? success "Output"

    A single [TimePeriod](../inputs_outputs/#timeperiod).

??? example
    {% raw %}
    ```YAML linenums="1"
    - active_directory.domain.lockout_duration:
      load:
        domain_controller: domain_controller
      save: lockout_duration

    - ppa.ui.output_info:
        text: Accounts will be locked out for {{ lockout_duration.days }} days & {{ lockout_duration.seconds }} seconds.
    ```
    {% endraw %}
"""
    with _catch_domain_object_errors():
        return domain.get_lockout_duration(domain_controller)
